// Copyright (c) 2001-2002 Rob Kaper <cap@capsi.com>,
//               2001 Erik Bourget <ebourg@cs.mcgill.ca>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// version 2 as published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; see the file COPYING.  If not, write to
// the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.

#include <stdlib.h>

#include "estate.h"
#include "estategroup.h"

Estate::Estate(int id, const std::string name, std::vector<Estate *> *ev)
	: GameObject(id, name, GameObject::GEstate)
{
	m_estates = ev;
	m_group = 0;
	m_payTarget = 0;
	m_owner = 0;
	m_takeCardGroup = 0;
	setHouses(0);
	setMortgaged(false);
	m_toJail = m_jail = false;
	m_mortgageprice = m_unmortgageprice = m_sellhouseprice = -1;
	m_payAmount = m_price = m_housePrice = m_money = m_passMoney = m_tax = m_taxPercentage = 0;
	m_color = m_bgColor = m_icon = "";
	m_rent[0] = m_rent[1] = m_rent[2] = m_rent[3] = m_rent[4] = m_rent[5] = 0;
	m_passMoney = 0;
}

int Estate::rent(Player *pLand, bool modifier)
{
	if (m_mortgaged)
		return 0;

	// Don't use m_housePrice, housePrice() also checks within estate group
	if (housePrice() > 0)
	{
		// Rent for estates that can have houses
		if (m_houses==0 && estateGroupIsMonopoly())
			return 2 * m_rent[m_houses];
		else
			return m_rent[m_houses];
	}

	// No custom rent rules, check group rules
	if (m_group)
	{
		int groupRent = m_group->rent(pLand);
		if (groupRent >= 0)
			return (modifier ? (2 * groupRent) : groupRent);
	}
	return 0;
}

int Estate::groupSize(Player *owner)
{
	int grouped = 0;

	Estate *eTmp = 0;
	for(std::vector<Estate *>::iterator it = m_estates->begin(); it != m_estates->end() && (eTmp = *it) ; ++it)
		if (m_group == eTmp->group() && m_owner == eTmp->owner())
			grouped++;

	return grouped;
}

int Estate::minHouses()
{
	int m_minHouses = 0;

	Estate *eTmp = 0;
	for(std::vector<Estate *>::iterator it = m_estates->begin(); it != m_estates->end() && (eTmp = *it) ; ++it)
	{
		if ((eTmp->group() == m_group) && (eTmp->houses() > m_minHouses))
			m_minHouses = eTmp->houses() - 1;
	}
	return m_minHouses;
}

int Estate::maxHouses()
{
	int m_maxHouses = 5;

	Estate *eTmp = 0;
	for(std::vector<Estate *>::iterator it = m_estates->begin(); it != m_estates->end() && (eTmp = *it) ; ++it)
	{
		if ((eTmp->group() == m_group) && (eTmp->houses() < m_maxHouses))
			m_maxHouses = eTmp->houses() + 1;
	}
	return m_maxHouses;
}

bool Estate::canBuyHouses(Player *p)
{
	if ( (p == m_owner) && estateGroupIsMonopoly() && !groupHasMortgages() && (m_houses < maxHouses()) )
		return true;
	else
		return false;
}

bool Estate::canSellHouses(Player *p) 
{
	if ( (p == m_owner) && estateGroupIsMonopoly() && (m_houses > minHouses()) )
		return true;
	else
		return false;
}

unsigned int Estate::price()
{
	return (!m_price && m_group) ? m_group->price() : m_price;
}

unsigned int Estate::housePrice()
{
	return (!m_housePrice && m_group) ? m_group->housePrice() : m_housePrice;
}

const std::string Estate::color()
{
	return m_color;
}

const std::string Estate::bgColor()
{
	return m_bgColor;
}

void Estate::setIcon(const std::string &icon)
{
	m_icon = icon;
}

std::string Estate::icon() const
{
	return m_icon;
}

bool Estate::canBeOwned()
{
	return (m_price || (m_group && m_group->price()));
}

bool Estate::canToggleMortgage(Player *p)
{
	return (p == m_owner && !groupHasBuildings()) ? true : false;
}

bool Estate::groupHasBuildings() 
{
	Estate *eTmp = 0;
	for(std::vector<Estate *>::iterator it = m_estates->begin(); it != m_estates->end() && (eTmp = *it) ; ++it) {
		if((eTmp->group() == m_group) && eTmp->houses())
			return true;
	}
	return false;
}

bool Estate::groupHasMortgages()
{
	Estate *eTmp = 0;
	for(std::vector<Estate *>::iterator it = m_estates->begin(); it != m_estates->end() && (eTmp = *it) ; ++it) {
		if((eTmp->group() == m_group) && eTmp->mortgaged())
			return true;
	}
	return false;
}

bool Estate::estateGroupIsMonopoly()
{
	unsigned int size = 0, sameOwner = 0;

	Estate *eTmp = 0;
	for(std::vector<Estate *>::iterator it = m_estates->begin(); it != m_estates->end() && (eTmp = *it) ; ++it)
		if (eTmp->group() == m_group)
		{
			size++;
			if (eTmp->owner() == m_owner)
				sameOwner++;
		}

	return (sameOwner == size);
}

void Estate::setMoney(int money)
{
	m_money = money;
}

int Estate::money()
{
	return m_money;
}

void Estate::addMoney(int money)
{
	m_money += money;
}

void Estate::setPayAmount(int amount)
{
	m_payAmount = amount;
}

int Estate::payAmount()
{
	return m_payAmount;
}
void Estate::setPassMoney(int passMoney)
{
	m_passMoney = passMoney;
}

int Estate::passMoney()
{
	return m_passMoney;
}

void Estate::setToJail(bool toJail)
{
	m_toJail = toJail;
}

bool Estate::toJail()
{
	return m_toJail;
}

void Estate::setJail(bool jail)
{
	m_jail = jail;
}

bool Estate::jail()
{
	return m_jail;
}

void Estate::setTakeCardGroup(CardGroup *takeCardGroup)
{
	m_takeCardGroup = takeCardGroup;
}

CardGroup *Estate::takeCardGroup()
{
	return m_takeCardGroup;
}
