// Copyright (c) 2001-2003 Rob Kaper <cap@capsi.com>,
//               2001 Erik Bourget <ebourg@cs.mcgill.ca>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// version 2 as published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; see the file COPYING.  If not, write to
// the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.

#define __USE_XOPEN
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <syslog.h>
#include <algo.h> // libstdc++ from the gcc 2.95 has no #include <algo> yet :(

#include <map>
#include <string>

#include "auction.h"
#include "card.h"
#include "cardgroup.h"
#include "debt.h"
#include "estate.h"
#include "estategroup.h"
#include "game.h"
#include "io.h"
#include "player.h"
#include "trade.h"

Game::Game(int id, const std::string &gameType)
	: GameObject(id)
{
	m_status = Config;

	m_minPlayers = 1;
	m_maxPlayers = 1;
	m_startMoney = 0;
	m_houses = 0;
	m_hotels = 0;
	m_master = m_pTurn = 0;
	m_goEstate =  0;
	m_description = "";
	m_gameType = m_bgColor = "";
	m_isValid = m_pausedForDialog = false;
	m_auction = 0;
	m_collectFines = m_doublePassMoney = false;
	m_auctionsEnabled = true;
	dice[0]=0;
	dice[1]=0;

	m_nextCardGroupId = m_nextEstateId = m_nextEstateGroupId = m_nextTradeId = m_nextAuctionId = 0;
}

Game::~Game()
{
	// We are responsible for the objects allocated
	while (!m_cardGroups.empty()) { delete *m_cardGroups.begin(); m_cardGroups.erase(m_cardGroups.begin()); }
	while (!m_debts.empty()) { delete *m_debts.begin(); m_debts.erase(m_debts.begin()); }
	while (!m_estates.empty()) { delete *m_estates.begin(); m_estates.erase(m_estates.begin()); }
	while (!m_estateGroups.empty()) { delete *m_estateGroups.begin(); m_estateGroups.erase(m_estateGroups.begin()); }
	while (!m_trades.empty()) { delete *m_trades.begin(); m_trades.erase(m_trades.begin()); }

	if (m_auction)
		delete m_auction;
}

void Game::ioWrite(const char *fmt, ...)
{
	int n, size = 256;
	char *buf = new char[size];
	static std::string ioStr;
	va_list arg;

	buf[0] = 0;

	while (1)
	{
		va_start(arg, fmt);
		n = vsnprintf(buf, size, fmt, arg);
		va_end(arg);

		if (n > -1 && n < size)
		{
			ioStr = buf;
			delete[] buf;
			ioWrite(ioStr);
			return;
		}

		if (n > -1)
			size = n+1;
		else
			size *= 2;

		delete[] buf;
		buf = new char[size];
	}
}

void Game::ioWrite(const std::string data)
{
	Player *pTmp = 0;
	for(std::vector<Player *>::iterator it = m_players.begin(); it != m_players.end() && (pTmp = *it) ; ++it)
		pTmp->ioWrite(data);
}

void Game::ioInfo(const char *data, ...)
{
	va_list arg;
	char buf[2048];
	
	va_start(arg, data);
	vsnprintf(buf, sizeof(buf)-1, data, arg);
	va_end(arg);
	buf[sizeof(buf)-1] = 0;

	ioWrite("<monopd><msg type=\"info\" value=\"%s\"/></monopd>\n", buf);
}

void Game::ioInfo(const std::string data)
{
	ioWrite("<monopd><msg type=\"info\" value=\"" + data + "\"/></monopd>\n");
}

void Game::ioError(const std::string data)
{
	ioWrite("<monopd><msg type=\"error\" value=\"" + data + "\"/></monopd>\n");
}

void Game::loadGameType(const std::string &gameType)
{
	if (m_gameType != gameType)
	{
		m_gameType = gameType;
		loadConfig();
	}
}

void Game::loadConfig()
{
	enum ConfigGroup { Board, EstateGroups, Estates, CardGroups, Cards, Other };
	ConfigGroup configGroup = Other;
	std::map<Estate *, int> payTargets;
	FILE *f;
	char str[256], *buf;
	EstateGroup *group = 0;
	Estate *eTmp = 0;
	CardGroup *cardGroup = 0;
	Card *card = 0;
	unsigned int goEstate = 0, nextCardId = 0;

	std::string filename = std::string(MONOPD_DATADIR "/games/") + m_gameType + ".conf";

	f = fopen(filename.c_str(), "r");
	if (!f)
	{
		f = fopen(MONOPD_DATADIR "games/atlantic.conf", "r");
		if (!f)
			return;
	}

	fgets(str, sizeof(str), f);
	while(!feof(f))
	{
		if (str[0]=='#') {}
		else if (str[0]=='<')
		{
			buf=strtok(str, ">\n\0");
			if (!strcasecmp(buf+1, "board"))
				configGroup = Board;
			else if (!strcasecmp(buf+1, "estategroups"))
				configGroup = EstateGroups;
			else if (!strcasecmp(buf+1, "estates"))
				configGroup = Estates;
			else if (!strcasecmp(buf+1, "cards"))
				configGroup = CardGroups;
			else
				configGroup = Other;
		}
		else if (str[0]=='[')
		{
			buf=strtok(str, "]\n\0");
			if (configGroup == EstateGroups)
				group = newGroup(buf+1);
			else if (configGroup == Estates)
				eTmp = newEstate(buf+1);
			else if (configGroup == Cards && cardGroup)
				card = cardGroup->newCard(nextCardId++, buf+1);
		}
		else if (strstr(str, "="))
		{
			buf = strtok(str, "=");
			if (!strcmp(buf, "name"))
				m_name = strtok(NULL, "\n\0");
			else if (!strcmp(buf, "description"))
				m_description = strtok(NULL, "\n\0");
			else if (!strcmp(buf, "minplayers"))
				m_minPlayers = atoi(strtok(NULL, "\n\0"));
			else if (!strcmp(buf, "maxplayers"))
				m_maxPlayers = atoi(strtok(NULL, "\n\0"));
			else if (!strcmp(buf, "houses"))
				m_houses = atoi(strtok(NULL, "\n\0"));
			else if (!strcmp(buf, "hotels"))
				m_hotels = atoi(strtok(NULL, "\n\0"));
			else if (!strcmp(buf, "startmoney"))
				m_startMoney = atoi(strtok(NULL, "\n\0"));
			else if (configGroup == Board)
			{
				if (!strcmp(buf, "go"))
					goEstate = atoi(strtok(NULL, "\n\0"));
				else if (!strcmp(buf, "bgcolor"))
					m_bgColor = strtok(NULL, "\n\0");
			}
			else if (configGroup == EstateGroups)
				parseConfigEntry(group, buf);
			else if (configGroup == Estates)
			{
				if (!strcmp(buf, "paytarget"))
					payTargets[eTmp] = atoi(strtok(NULL, "\n\0"));
				else
					parseConfigEntry(eTmp, buf);
			}
			else if (configGroup == CardGroups)
			{
				if (!strcmp(buf, "groupname"))
				{
					cardGroup = newCardGroup(strtok(NULL, "\n\0"));
					configGroup = Cards;
				}
			}
			else if (configGroup == Cards)
			{
				if (!strcmp(buf, "groupname"))
					cardGroup = newCardGroup(strtok(NULL, "\n\0"));
				else
					parseConfigEntry(card, buf);
			}
		}
		fgets(str, sizeof(str), f);
	}
	fclose(f);

	// Make sure defaults are within reasonable limits
	if (m_minPlayers < 2)
		m_minPlayers = 2;
	if (m_maxPlayers > 32)
		m_maxPlayers = 32;

	// Go estate is required, defaults to first specified
	if (!(m_goEstate = findEstate(goEstate)))
		m_goEstate = findEstate(0);
	if (!m_goEstate)
		return;

	// Fix defaults for estates
	for (std::map<Estate *, int>::iterator it = payTargets.begin() ; it != payTargets.end() && (eTmp = (*it).first) ; it++)
		eTmp->setPayTarget(findEstate((*it).second));
	payTargets.clear();

	eTmp = 0;
	for (std::vector<Estate *>::iterator eIt = m_estates.begin() ; eIt != m_estates.end() && (eTmp = *eIt) ; eIt++)
	{
		if (eTmp->mortgagePrice() < 0)
			eTmp->setMortgagePrice((int)(eTmp->price()/2));
		if (eTmp->unmortgagePrice() < 0)
			eTmp->setUnmortgagePrice((int)((eTmp->price()/2)*1.1));
		if (eTmp->sellHousePrice() < 0)
			eTmp->setSellHousePrice((int)(eTmp->housePrice()/2));
	}

	m_isValid = true;
}

void Game::parseConfigEntry(Estate *es, char *buf)
{
	if (!es)
		return;

	bool mortgageprice_set, unmortgageprice_set, sellhouseprice_set;

	sellhouseprice_set = mortgageprice_set = unmortgageprice_set = false;

	if (!strcmp(buf, "group"))
		es->setEstateGroup( findGroup(strtok(NULL, "\n\0")) );
	else if (!strcmp(buf, "color"))
		es->setColor(strtok(NULL, "\n\0"));
	else if (!strcmp(buf, "bgcolor"))
		es->setBgColor(strtok(NULL, "\n\0"));
	else if (!strcmp(buf, "icon"))
		es->setIcon(strtok(NULL, "\n\0"));
	else if (!strcmp(buf, "price"))
		es->setPrice(atoi(strtok(NULL, "\n\0")));
	else if (!strcmp(buf, "mortgageprice")) 
	{
		es->setMortgagePrice(atoi(strtok(NULL, "\n\0")));
		mortgageprice_set = true;
	}
	else if (!strcmp(buf, "unmortgageprice")) 
	{
		es->setUnmortgagePrice(atoi(strtok(NULL, "\n\0")));
		unmortgageprice_set = true;
	}
	else if (!strcmp(buf, "houseprice"))
		es->setHousePrice(atoi(strtok(NULL, "\n\0")));
	else if (!strcmp(buf, "sellhouseprice")) 
	{
		es->setSellHousePrice(atoi(strtok(NULL, "\n\0")));
		sellhouseprice_set = true;
	}
	else if (!strcmp(buf, "tax"))
		es->setTax(atoi(strtok(NULL, "\n\0")));
	else if (!strcmp(buf, "taxpercentage"))
		es->setTaxPercentage(atoi(strtok(NULL, "\n\0")));
	else if (!strcmp(buf, "rent0"))
		es->setRent(0, atoi(strtok(NULL, "\n\0")));
	else if (!strcmp(buf, "rent1"))
		es->setRent(1, atoi(strtok(NULL, "\n\0")));
	else if (!strcmp(buf, "rent2"))
		es->setRent(2, atoi(strtok(NULL, "\n\0")));
	else if (!strcmp(buf, "rent3"))
		es->setRent(3, atoi(strtok(NULL, "\n\0")));
	else if (!strcmp(buf, "rent4"))
		es->setRent(4, atoi(strtok(NULL, "\n\0")));
	else if (!strcmp(buf, "rent5"))
		es->setRent(5, atoi(strtok(NULL, "\n\0")));
	else if (!strcmp(buf, "tojail"))
		es->setToJail(atoi(strtok(NULL, "\n\0")));
	else if (!strcmp(buf, "jail"))
		es->setJail(atoi(strtok(NULL, "\n\0")));
	else if (!strcmp(buf, "payamount"))
		es->setPayAmount(atoi(strtok(NULL, "\n\0")));
	else if (!strcmp(buf, "passmoney"))
		es->setPassMoney(atoi(strtok(NULL, "\n\0")));
	else if (!strcmp(buf, "takecard"))
		es->setTakeCardGroup( findCardGroup(strtok(NULL, "\n\0")) );
}

void Game::parseConfigEntry(Card *card, char *buf)
{
	if (!card)
		return;

	if (!strcmp(buf, "canbeowned"))
		card->setCanBeOwned(atoi(strtok(NULL, "\n\0")));
	else if (!strcmp(buf, "pay"))
		card->setPay(atoi(strtok(NULL, "\n\0")));
	else if (!strcmp(buf, "payhouse"))
		card->setPayHouse(atoi(strtok(NULL, "\n\0")));
	else if (!strcmp(buf, "payhotel"))
		card->setPayHotel(atoi(strtok(NULL, "\n\0")));
	else if (!strcmp(buf, "payeach"))
		card->setPayEach(atoi(strtok(NULL, "\n\0")));
	else if (!strcmp(buf, "tojail"))
		card->setToJail(atoi(strtok(NULL, "\n\0")));
	else if (!strcmp(buf, "outofjail"))
		card->setOutOfJail(atoi(strtok(NULL, "\n\0")));
	else if (!strcmp(buf, "advance"))
		card->setAdvance(atoi(strtok(NULL, "\n\0")));
	else if (!strcmp(buf, "advanceto"))
		card->setAdvanceTo(atoi(strtok(NULL, "\n\0")));
	else if (!strcmp(buf, "advancetonextof"))
		card->setAdvanceToNextOf(findGroup(strtok(NULL, "\n\0")));
}

void Game::parseConfigEntry(EstateGroup *group, char *buf)
{
	 if (!group)
	 	return;

	if (!strcmp(buf, "price"))
		group->setPrice(atoi(strtok(NULL, "\n\0")));
	else if (!strcmp(buf, "houseprice"))
		group->setHousePrice(atoi(strtok(NULL, "\n\0")));
	else if (!strcmp(buf, "color"))
		group->setColor(strtok(NULL, "\n\0"));
	else if (!strcmp(buf, "bgcolor"))
		group->setBgColor(strtok(NULL, "\n\0"));
	else if (!strcmp(buf, "rentmath"))
		group->setRentMath(strtok(NULL, "\n\0"));
	else if (!strcmp(buf, "rentvar"))
		group->setRentVar(strtok(NULL, "\n\0"));
}

const std::string Game::description()
{
	if (m_description == "" && m_master)
		return m_master->name() + "'s game";
	else
		return escapeXML(m_description);
}

void Game::editConfig(Player *pInput, char *data)
{
	if (pInput != m_master)
	{
		pInput->ioError("Only the master can edit the game configuration.");
		return;
	}
	if (m_status != Config)
	{
		pInput->ioError("This game has already been started.");
		return;
	}

	switch(data[0])
	{
		case 'a':
			setAuctionsEnabled(atoi(data+1));
			break;
		case 'f':
			setCollectFines(atoi(data+1));
			break;
		default:
			pInput->ioError("No such game configuration command.");
	}
}

void Game::start(Player *pInput)
{
	if (pInput != m_master)
	{
		pInput->ioError("Only the master can start a game.");
		return;
	}
	if (m_status != Config)
	{
		pInput->ioError("This game has already been started.");
		return;
	}
	if (m_players.size() < m_minPlayers)
	{
		pInput->ioError("This game requires at least %d players to be started.", m_minPlayers);
		return;
	}

	// Update status.
	m_status = Init;

	// Shuffle players
	random_shuffle(m_players.begin(), m_players.end());

	// Shuffle card decks.
	CardGroup *cardGroup = 0;
	for (std::vector<CardGroup *>::iterator it = m_cardGroups.begin() ; it != m_cardGroups.end() && (cardGroup = *it) ; ++it)
		cardGroup->shuffleCards();

	// Set all players at Go, give them money
	Player *pTmp = 0;
	for(std::vector<Player *>::iterator it = m_players.begin(); it != m_players.end() && (pTmp = *it) ; ++it)
	{
		pTmp->reset();
		pTmp->setEstate(m_goEstate);
		pTmp->addMoney(m_startMoney);
	}

	for(std::vector<Player *>::iterator it = m_players.begin(); it != m_players.end() && (pTmp = *it) ; ++it)
		sendFullUpdate(pTmp);

	// Add notification of game start.
	m_status = Run;
	ioWrite("<monopd><gameupdate gameid=\"%d\" status=\"%s\"/></monopd>\n", m_id, statusLabel().c_str());

	// Turn goes to first player
	m_pTurn = *m_players.begin();
	m_pTurn->setHasTurn(true);
}

void Game::setTokenLocation(Player *pInput, unsigned int estateId)
{
	bool useNext = false;
	unsigned int money = 0;

	if (!pInput->tokenLocation())
		return;

	Estate *estate = 0;
	for (std::vector<Estate *>::iterator it = m_estates.begin() ;;)
	{
		if (it == m_estates.end())
		{
			it = m_estates.begin();
			continue;
		}
		if (!(estate = *it))
			break;

		if (useNext)
		{
			if (estate->passMoney())
			{
				pInput->setDisplay(0, "%s passes %s and gets %d.", m_pTurn->name().c_str(), estate->name().c_str(), estate->passMoney());
				pInput->sendDisplayMsg();
				money += estate->passMoney();
				pInput->ioWrite("<monopd><playerupdate playerid=\"%d\" money=\"%d\"/></monopd>\n", m_pTurn->id(), m_pTurn->money() + money);
			}
			if (estate == m_pTurn->destination())
			{
				pInput->setTokenLocation(0); // Player is done moving
				break;
			}
			if (estate->id() == estateId)
			{
				pInput->setTokenLocation(estate);
				break;
			}
		}
		else if (estate == pInput->tokenLocation())
			useNext = true;

		++it;
	}

	// Find out if there are still clients busy with movement.
	if (clientsMoving()) 
		return;

	// Land player!
	bool endTurn = landPlayer(m_pTurn, false);

	if (clientsMoving())
	{
		// Don't do a thing.. just wait until they are done.
	}
	else if (m_pTurn->canBuy())
	{
		// Don't do a thing..
	}
	else if (endTurn)
		m_pTurn->endTurn();
}

void Game::tokenMovementTimeout()
{
	// Mark all clients as non moving.
	Player *pTmp = 0;
	for(std::vector<Player *>::iterator it = m_players.begin(); it != m_players.end() && (pTmp = *it) ; ++it)
		if (pTmp->tokenLocation())
		{
			if (m_pTurn && m_pTurn->destination())
			{
				pTmp->ioWrite("<monopd><playerupdate playerid=\"%d\" location=\"%d\" directmove=\"1\"/></monopd>\n", m_pTurn->id(), m_pTurn->destination()->id());
				setTokenLocation(pTmp, m_pTurn->destination()->id());
			}
		}
}

unsigned int Game::auctionTimeout()
{
	if (!m_auction)
		return 0;

	m_auction->setStatus(m_auction->status()+1);
	ioWrite("<monopd><auctionupdate auctionid=\"%d\" status=\"%d\"/></monopd>\n", m_auction->id(), m_auction->status());
	if (m_auction->status() == Auction::Sold)
	{
		completeAuction();
		delAuction();
		return 0;
	}
	return 4;
}

void Game::newDebtToAll(Player *from, int amount)
{
	Player *pTmp = 0;
	for(std::vector<Player *>::iterator it = m_players.begin() ; it != m_players.end() && (pTmp = *it) ; ++it)
		if (pTmp != from)
			newDebt(from, pTmp, 0, amount);
}

void Game::newDebt(Player *from, Player *toPlayer, Estate *toEstate, int amount)
{
	if (from->bankrupt())
		return;

	Debt *d = new Debt(from, toPlayer, toEstate, amount);
	m_debts.push_back(d);

	if (amount > from->assets())
	{
		setDisplay(0, false, false, "%s is bankrupt!", from->name().c_str());
		bankruptPlayer(from);
	}
}

Debt *Game::findDebt(Player *_p)
{
	Debt *dTmp = 0;
	for(std::vector<Debt *>::iterator it = m_debts.begin(); it != m_debts.end() && (dTmp = *it) ; ++it)
		if (_p == dTmp->from())
			return dTmp;

	return NULL;
}

bool Game::hasDebt() 
{
	for(std::vector<Debt *>::iterator it = m_debts.begin(); it != m_debts.end(); ++it)
		if (*it)
			return true;
	return false;
}

void Game::delDebt(Debt *_d)
{
	for(std::vector<Debt *>::iterator it = m_debts.begin(); it != m_debts.end(); ++it)
		if (*it == _d)
		{
			m_debts.erase(it);
			break;
		}
	delete _d;
}

void Game::solveDebt(Player *pInput, const bool verbose)
{
	Debt *debt = findDebt(pInput);
	if (!debt)
	{
		if (verbose)
			ioError("You don't have any debts to pay off!");
		return;
	}

	int payAmount = debt->amount();
	if ( !(pInput->payMoney(payAmount)) )
	{
		if (verbose)
			pInput->ioError("You're not solvent yet. You owe %d and have %d.", payAmount, pInput->money());
		return;
	}

	Player *pCreditor = 0;
	Estate *eCreditor = 0;
	if ((pCreditor = debt->toPlayer()))
		pCreditor->addMoney(payAmount);
	else if ((eCreditor = debt->toEstate()))
		eCreditor->addMoney(payAmount);

	ioInfo("%s pays off a %d debt to %s.", pInput->name().c_str(), debt->amount(), (pCreditor ? pCreditor->name().c_str() : "Bank"));
	delDebt(debt);

	if (unsigned int debts = m_debts.size())
	{
		ioInfo("There are still %d debts, game still paused.", debts);
		ioInfo("All debts are settled, game continues.");
	}
}

void Game::newAuction(Player *pInput) 
{
	if (!m_auctionsEnabled)
	{
		pInput->ioError("Auctions are not enabled.");
		return;
	}
	if (!(pInput->canBuy()))
	{
		pInput->ioError("You cannot auction anything at the moment.");
		return;
	}

	Estate *estate = pInput->estate();

	m_auction = new Auction();
	m_auction->setEstate(estate);

	pInput->setCanBuy(false);
	ioWrite("<monopd><auctionupdate auctionid=\"%d\" actor=\"%d\" estateid=\"%d\" status=\"0\"/></monopd>\n", m_auction->id(), pInput->id(), estate->id());
}

Auction *Game::auction()
{
	return m_auction;
}

void Game::delAuction()
{
	delete m_auction;
	m_auction = 0;
}

// Returns 0 on succesful bid, 1 on error.
int Game::bidInAuction(Player *pInput, char *data)
{
	if (!strstr(data, ":"))
	{
		pInput->ioError("Invalid input for .ab, no seperator after auctionId");
		return 1;
	}
	int auctionId = atoi(strsep(&data, ":"));
	int bid = atoi(data);

	if (m_auction->id() != auctionId)
	{
		pInput->ioError("No such auctionId: %d.", auctionId);
		return 1;
	}

	if (bid > pInput->assets())
	{
		pInput->ioError("You don't have %d.", bid);
		return 1;
	}
	
	if (bid <= m_auction->highBid())
	{
		pInput->ioError("Minimum bid is %d.", m_auction->highBid()+1);
		return 1;
	}

	m_auction->setHighBid(pInput, bid);
	ioWrite("<monopd><auctionupdate auctionid=\"%d\" highbid=\"%d\" highbidder=\"%d\" status=\"%d\"/></monopd>\n", m_auction->id(), bid, pInput->id(), m_auction->status());
	return 0;
}

void Game::newTrade(Player *pInput, unsigned int playerId)
{
	Player *player = findPlayer(playerId);
	if (!player)
	{
		pInput->ioError("No such playerId: %ld.", playerId);
		return;
	}
	else if (player == pInput)
	{
		pInput->ioError("Trading with yourself?");
		return;
	}

	// Create a new trade between us and player
	Trade *t = new Trade(pInput, player, m_nextTradeId++);
	m_trades.push_back(t);
}

Trade *Game::findTrade(Player *_p)
{
	Trade *tTmp = 0;
	for(std::vector<Trade *>::iterator it = m_trades.begin(); it != m_trades.end() && (tTmp = *it) ; ++it)
		if (tTmp->hasPlayer(_p))
			return tTmp;

	return 0;
}

Trade *Game::findTrade(unsigned int tradeId)
{
	Trade *tTmp = 0;
	for(std::vector<Trade *>::iterator it = m_trades.begin(); it != m_trades.end() && (tTmp = *it) ; ++it)
		if (tradeId == tTmp->id()) return tTmp;
	
	return 0;
}

void Game::delTrade(Trade *trade)
{
	for(std::vector<Trade *>::iterator it = m_trades.begin(); it != m_trades.end(); ++it)
		if (*it == trade)
		{
			delete trade;
			m_trades.erase(it);
			break;
		}
}

void Game::acceptTrade(Player *pInput, unsigned int tradeId)
{
	Trade *trade = findTrade(tradeId);
	if (!trade)
	{
		pInput->ioError("No such tradeId: %ld.", tradeId);
		return;
	}
	if ( !(trade->hasPlayer(pInput)) )
	{
		pInput->ioError("You are not part of trade %ld.", tradeId);
		return;
	}

	trade->setPlayerAccept(pInput, true);

	if (trade->allAccept())
	{
		// Complete trade
		completeTrade(trade);
		delTrade(trade);
		m_pTurn->endTurn();
	}
}

void Game::rejectTrade(Player *pInput, unsigned int tradeId)
{
	Trade *trade = findTrade(tradeId);
	if (!trade)
	{
		pInput->ioError("No such tradeId: %d.", tradeId);
		return;
	}

	if ( !(trade->hasPlayer(pInput)) )
	{
		pInput->ioError("You are not part of trade %d.", tradeId);
		return;
	}

	ioWrite("<monopd><tradeupdate type=\"rejected\" tradeid=\"%d\" actor=\"%d\"/></monopd>\n", trade->id(), pInput->id());
	delTrade(trade);
}

void Game::completeTrade(Trade *trade)
{
	ioWrite("<monopd><tradeupdate tradeid=\"%d\" type=\"accepted\"/></monopd>\n", trade->id());

	Player *pFrom, *pTo;
	GameObject *object;
	while((pTo = trade->firstTarget()))
	{
		pFrom = trade->firstFrom();
		object = trade->takeFirstObject();
		switch(object->type())
		{
		case GameObject::Money:
			unsigned int money;
			money = object->id();
			delete object; // was temporarily created to serve as trade object
			if (!pFrom->payMoney(money))
			{
				ioInfo("%s owes %d to %s. Game paused, %s is not solvent. Player needs to raise %d in cash first.", pFrom->name().c_str(), money, pTo->name().c_str(), money, pFrom->name().c_str(), (money - pFrom->money()));
				newDebt(pFrom, pTo, 0, money);
			}
			else
				pTo->addMoney(money);

			ioInfo("%s gets %d from %s.", pTo->name().c_str(), money, pFrom->name().c_str());
			break;
		default:
			transferObject(object->type(), object->id(), pTo, true);
		}
	}

 	// playerupdate messages for trade players
	Player *player = 0;
	for(std::vector<Player *>::iterator it = m_players.begin(); it != m_players.end() && (player = *it) ; ++it)
		if (trade->hasPlayer(player))
			ioWrite("<monopd><playerupdate playerid=\"%d\" money=\"%d\"/></monopd>\n", player->id(), player->money());

	// Write "completed" to all players
	ioWrite("<monopd><tradeupdate tradeid=\"%d\" type=\"completed\"/></monopd>\n", trade->id());

	// Solve debts for trade players
	for(std::vector<Player *>::iterator it = m_players.begin(); it != m_players.end() && (player = *it) ; ++it)
		if (trade->hasPlayer(player))
			solveDebt(player);
}

void Game::completeAuction()
{
	Player *pBid = m_auction->highBidder();
	if (!pBid)
		return;

	unsigned int bid = m_auction->highBid();

	if (!pBid->payMoney(bid))
	{
		ioInfo("%s has to pay %d. Game paused, %s is not solvent. Player needs to raise %d in cash first.", pBid->name().c_str(), bid, pBid->name().c_str(), (bid - pBid->money()));
		newDebt(pBid, 0, 0, bid);
	}

	Estate *estate = m_auction->estate();
	transferEstate(estate, pBid);
	setDisplay(estate, false, false, "Purchased by %s in an auction for %d.", pBid->name().c_str(), bid);

	m_pTurn->setCanBuy(false);
	if (m_pTurn->canRollAgain())
	{
		m_pTurn->setCanRollAgain(false);
		m_pTurn->setCanRoll(true);
		setDisplay(estate, false, false, "%s may roll again.", m_pTurn->name().c_str());
	}
	else 
		updateTurn();
}

void Game::setDisplay(Estate *estate, bool clearText, bool clearButtons, const char *data, ...)
{
	va_list arg;
	char buf[2048];
	
	va_start(arg, data);
	vsnprintf(buf, sizeof(buf)-1, data, arg);
	va_end(arg);
	buf[sizeof(buf)-1] = 0;

	Player *player = 0;
	for (std::vector<Player *>::iterator pit = m_players.begin(); pit != m_players.end() && (player = *pit) ; ++pit)
	{
		player->setDisplay(estate, std::string(buf));
		player->setDisplayClearText(clearText);
		player->setDisplayClearButtons(clearButtons);
		player->sendDisplayMsg();
	}
}

void Game::sendMsgEstateUpdate(Estate *e) 
{
	Estate *eTmp = 0;
	Player *pTmp = 0;
	for (std::vector<Player *>::iterator pit = m_players.begin(); pit != m_players.end() && (pTmp = *pit) ; ++pit) {
		for(std::vector<Estate *>::iterator eit = m_estates.begin(); eit != m_estates.end() && (eTmp = *eit) ; ++eit) {
			if(eTmp->group() == e->group())
				pTmp->ioWrite("<monopd><estateupdate estateid=\"%d\" owner=\"%d\" can_toggle_mortgage=\"%d\" can_buy_houses=\"%d\" can_sell_houses=\"%d\" mortgaged=\"%d\" houses=\"%d\"/></monopd>\n", eTmp->id(), (eTmp->owner() ? eTmp->owner()->id() : -1), eTmp->canToggleMortgage(pTmp), eTmp->canBuyHouses(pTmp), eTmp->canSellHouses(pTmp), eTmp->mortgaged(), eTmp->houses());
		}
	}
}

EstateGroup *Game::newGroup(char *n)
{
	EstateGroup *group = new EstateGroup(m_nextEstateGroupId++, this, n);
	m_estateGroups.push_back(group);
	return group;
}

EstateGroup *Game::findGroup(const std::string name)
{
	EstateGroup *group = 0;
	for(std::vector<EstateGroup *>::iterator it = m_estateGroups.begin(); it != m_estateGroups.end() && (group = *it) ; ++it)
		if (group->name() == name)
			return group;

	return 0;
}

unsigned int Game::estateGroupSize(EstateGroup *estateGroup)
{
	unsigned int size = 0;

	Estate *eTmp = 0;
	for(std::vector<Estate *>::iterator it = m_estates.begin(); it != m_estates.end() && (eTmp = *it) ; ++it)
		if (estateGroup == eTmp->group())
			size++;

	return size;
}

Estate *Game::newEstate(const std::string name)
{
	Estate *es = new Estate(m_nextEstateId++, name, &m_estates);
	m_estates.push_back(es);
	return es;
}

Estate *Game::findEstate(int id)
{
	if (id >= 0 && id < (int)m_estates.size())
		return m_estates[id];
	else
		return NULL;
}

Estate *Game::findNextEstate(EstateGroup *group, Estate *startEstate)
{
	bool useNext = true;
	if (startEstate)
		useNext = false;

	Estate *eTmp = 0, *eFirst = 0;
	for (std::vector<Estate *>::iterator eIt = m_estates.begin() ; eIt != m_estates.end() && (eTmp = *eIt) ; eIt++)
	{
		if (startEstate && startEstate == eTmp)
			useNext = true;
		if (eTmp->group() == group)
		{
			if (useNext==true)
				return eTmp;
			else if (!eFirst)
				eFirst = eTmp;
		}
	}
	return eFirst;
}

Estate *Game::findNextJailEstate(Estate *startEstate)
{
	bool useNext = true;
	if (startEstate)
		useNext = false;

	Estate *eTmp = 0, *eFirst = 0;
	for (std::vector<Estate *>::iterator eIt = m_estates.begin() ; eIt != m_estates.end() && (eTmp = *eIt) ; eIt++)
	{
		if (startEstate && startEstate == eTmp)
			useNext = true;
		if (eTmp->jail())
		{
			if (useNext==true)
				return eTmp;
			else if (!eFirst)
				eFirst = eTmp;
		}
	}
	return eFirst;
}

void Game::delEstate(Estate *_e)
{
	for(std::vector<Estate *>::iterator it = m_estates.begin(); it != m_estates.end(); ++it)
		if (*it == _e)
		{
			m_estates.erase(it);
			break;
		}
	delete _e;
}

void Game::transferEstate(Estate *estate, Player *player, const bool verbose)
{
	if (estate->owner())
	{
		Trade *trade = 0;
		for(std::vector<Trade *>::iterator it = m_trades.begin(); it != m_trades.end() && (trade = *it) ; ++it)
			trade->delComponent(estate->type(), estate->id());
	}

	estate->setOwner(player);
	sendMsgEstateUpdate(estate);

	if (verbose)
		ioInfo("%s is now the owner of estate %s.", player->name().c_str(), estate->name().c_str());
}

void Game::transferObject(const enum GameObject::Type type, unsigned int id, Player *player, const bool verbose)
{
	switch(type)
	{
	case GameObject::GCard:
		transferCard(findCard(id), player, verbose);
		break;
	case GameObject::GEstate:
		transferEstate(findEstate(id), player, verbose);
		break;
	default:;
	}
}

void Game::setPausedForDialog(const bool paused)
{
	m_pausedForDialog = paused;
}

const bool Game::pausedForDialog()
{
	return m_pausedForDialog;
}

void Game::transferCard(Card *card, Player *player, const bool verbose)
{
	Player *owner = card->owner();
	if (owner)
	{
		owner->takeCard(card);
		card->setOwner(0);
		Trade *trade = 0;
		for(std::vector<Trade *>::iterator it = m_trades.begin(); it != m_trades.end() && (trade = *it) ; ++it)
			trade->delComponent(card->type(), card->id());
	}
	else
		card->group()->popCard();

	if (player)
	{
		player->addCard(card);
		card->setOwner(player);
	}
	else
		card->group()->pushCard(card);

	ioWrite("<monopd><cardupdate cardid=\"%d\" owner=\"%d\"/></monopd>\n", card->id(), card->owner() ? card->owner()->id() : -1);
	if (verbose)
		ioInfo("%s is now the owner of card %d.", player->name().c_str(), card->id());
}

CardGroup *Game::newCardGroup(const std::string name)
{
	CardGroup *cardGroup = new CardGroup(m_nextCardGroupId++, this, name);
	m_cardGroups.push_back(cardGroup);
	return cardGroup;
}

CardGroup *Game::findCardGroup(const std::string name)
{
	CardGroup *cardGroup = 0;
	for(std::vector<CardGroup *>::iterator it = m_cardGroups.begin(); it != m_cardGroups.end(); ++it)
		if ((cardGroup = *it) && cardGroup->name() == name)
			return cardGroup;

	return 0;
}

Card *Game::findCard(unsigned int id)
{
	Card *card = 0;

	CardGroup *cardGroup = 0;
	for(std::vector<CardGroup *>::iterator it = m_cardGroups.begin(); it != m_cardGroups.end() && (cardGroup = *it) ; ++it)
	{
		card = cardGroup->findCard(id);
		if (card)
			return card;
	}

	Player *player = 0;
	for(std::vector<Player *>::iterator it = m_players.begin(); it != m_players.end() && (player = *it) ; ++it)
	{
		card = player->findCard(id);
		if (card)
			return card;
	}

	return 0;
}

Player *Game::addPlayer(Player *p, bool isMaster)
{
	m_players.push_back(p);
	p->setGame(this);

	if (isMaster)
		m_master = p;

	// Inform player of join.
	sendStatus(p);
	sendConfiguration(p);

	// Inform all players of join.
	// FIXME: DEPRECATED
	ioWrite("<monopd><updateplayerlist type=\"add\"><player playerid=\"%ld\" name=\"%s\" host=\"%s\" master=\"%d\"/></updateplayerlist></monopd>\n", p->id(), p->name().c_str(), p->host().c_str(), isMaster);

	ioWrite("<monopd><playerupdate playerid=\"%ld\" game=\"%d\" name=\"%s\" host=\"%s\" master=\"%d\"/></monopd>\n", p->id(), m_id, p->name().c_str(), p->host().c_str(), isMaster);

	return p;
}

Player *Game::findPlayer(int id)
{
	Player *pTmp = 0;
	for(std::vector<Player *>::iterator it = m_players.begin(); it != m_players.end() && (pTmp = *it) ; ++it)
		if(id == pTmp->id())
			return pTmp;
	
	return 0;
}

void Game::removePlayer(Player *p)
{
	bool newMaster = false;

	p->setGame(0);
	if (p==master())
		newMaster=true;

	// Find player and erase from std::vector
	for(std::vector<Player *>::iterator it = m_players.begin(); it != m_players.end(); ++it)
		if (*it == p)
		{
			m_players.erase(it);
			break;
		}

	// XML message to remove player from list
	// FIXME: DEPRECATED
	ioWrite("<monopd><updateplayerlist type=\"del\"><player playerid=\"%ld\"/></updateplayerlist></monopd>\n", p->id());

	ioWrite("<monopd><playerupdate playerid=\"%d\" game=\"-1\"/></monopd>\n", p->id());

	if (newMaster && m_players.size() > 0)
	{
		m_master = m_players[0];

		// XML message to set new master
		// FIXME: DEPRECATED
		ioWrite("<monopd><updateplayerlist type=\"edit\"><player playerid=\"%ld\" master=\"1\"/></updateplayerlist></monopd>\n", m_master->id());

		ioWrite("<monopd><playerupdate playerid=\"%ld\" master=\"1\"/></monopd\n", m_master->id());
		if (m_status == Config)
			sendConfiguration(m_master);
	}
}

unsigned int Game::playerAssets(Player *player)
{
	unsigned int assets = player->money();
	Estate *estate = 0;
	for(std::vector<Estate *>::iterator it = m_estates.begin(); it != m_estates.end() && (estate = *it) ; ++it)
	{
		if ( player == estate->owner() )
		{
			if (!estate->mortgaged())
			{
				assets += estate->mortgagePrice();
				assets += ( estate->houses() * estate->sellHousePrice() );
			}
		}
	}
	return assets;
}

void Game::updateTurn()
{
	Player *pOldTurn = m_pTurn;
	// Disable turn, roll and buy.
	pOldTurn->setHasTurn(false);

	Player *player = 0, *pFirst = 0;
	bool useNext = false;
	for(std::vector<Player *>::iterator it = m_players.begin() ; it != m_players.end() && (player = *it) ; ++it)
	{
		if (!pFirst && !player->bankrupt())
			pFirst = player;
			
		if (player == pOldTurn)
			useNext = true;
		else if (useNext && !player->bankrupt())
		{
			m_pTurn = player;
			useNext = false;
			break;
		}
	}
	if (useNext && pFirst)
		m_pTurn = pFirst;

	// Set turn.
	setDisplay(pOldTurn->estate(), false, false, "Turn goes to %s.", m_pTurn->name().c_str());
	m_pTurn->setHasTurn(true);
}

bool Game::landPlayer(Player *pTurn, const bool directMove)
{
	Estate *destination = pTurn->destination();

	bool useNext = false;
	unsigned int money = 0;
	Estate *estate = 0;
	for (std::vector<Estate *>::iterator it = m_estates.begin() ;;)
	{
		if (it == m_estates.end())
		{
			it = m_estates.begin();
			continue;
		}

		if (!(estate = *it))
			break;

		if (useNext)
		{
			if (estate->passMoney())
			{
				money += estate->passMoney();
				// Write incremental message for direct moves, token
				// confirmation or timeout didn't do it yet.
				if (directMove)
					ioInfo("%s passes %s and gets %d.", pTurn->name().c_str(), estate->name().c_str(), estate->passMoney());
			}
			if (estate == destination)
				break;
		}
		else if (estate == pTurn->estate())
			useNext = true;

		++it;
	}

	pTurn->setEstate(destination);
	pTurn->setDestination(0);

	// Store and write final data.
	pTurn->addMoney(money);
	
	// Hm, we wait for all tokens to have landed, this is exactly what we
	// were trying to avoid with async updates! So some of this stuff can be
	// sent earlier.

	// What properties does the estate we are landing on have?
	bool endTurn = true;
	Player *pOwner = 0;
	Estate *es = pTurn->estate();

	setDisplay(es, false, false, "%s lands on %s.", pTurn->name().c_str(), es->name().c_str());

	// toJail is really serious: if an estate has this property, all other
	// properties are ignored when landing it. TODO: Don't ignore and allow
	// weird combinations?
	if ( es->toJail() )
	{
		if ( Estate *eTmp = findNextJailEstate(es) )
			pTurn->toJail(eTmp);
		else
			ioError("This gameboard does not have a jail estate.");

		updateTurn();
		return true;
	}

	// Any estate can have a pot of gold. This is handled before all other
	// properties except toJail and works in combination with any of those
	// properties.
	if ( int estateMoney = es->money() )
	{
		if ( estateMoney > 0)
		{
			// Good for you!
			pTurn->addMoney(estateMoney);
			es->setMoney(0);
			ioWrite("<monopd><estateupdate estateid=\"%d\" money=\"%d\"/></monopd>\n", es->id(), es->money());
			setDisplay(es, false, false, "%s gets %d from the fine pot.", pTurn->name().c_str(), estateMoney);
		}
		else
		{
			// Pay up!
			if (!pTurn->payMoney(estateMoney))
			{
				// TODO: If we go into debt here, we'll never enter rent
				// calculation. But we'll need negative money() piles for
				// Estates because that's basically what tax estates are.
				ioInfo("%s has to pay %d. Game paused, %s is not solvent. Player needs to raise %d in cash first.", pTurn->name().c_str(), estateMoney, pTurn->name().c_str(), (estateMoney - pTurn->money()));
				newDebt(pTurn, 0, es->payTarget(), estateMoney);
				return true;
			}
			else if (es->payTarget() && m_collectFines)
				es->payTarget()->addMoney(estateMoney);

			es->setMoney(0);
			ioWrite("<monopd><estateupdate estateid=\"%d\" money=\"%d\"/></monopd>\n", es->id(), es->money());
			setDisplay(es, false, false, "%s pays %d.", pTurn->name().c_str(), estateMoney);
		}
	}

	// Any estate can have taxes on them. This is handled before rent and purchase opportunities.
	// TODO: Merge with money() stuff to avoid duplication?
	if ( es->tax() || es->taxPercentage() )
	{
		int payAmount, tax, taxPercentage;
		tax = es->tax();
		taxPercentage = es->taxPercentage();

		if (tax && taxPercentage)
		{
			Player *pTmp = 0;
			for (std::vector<Player *>::iterator it = m_players.begin() ; it != m_players.end() && (pTmp = *it) ; ++it)
				if (pTmp != pTurn)
				{
					pTmp->setDisplay(es, "%s must pay either %d or %d percent of his/her assets.", pTurn->name().c_str(), tax, taxPercentage);
					pTmp->sendDisplayMsg();
				}

			pTurn->setDisplay(es, "Pay either %d or %d percent of your assets.", tax, taxPercentage);
			pTurn->addDisplayButton(".T$", std::string("Pay ") + itoa(tax), 1);\
			pTurn->addDisplayButton(".T%", std::string("Pay ") + itoa(taxPercentage) + " Percent", 1);
			pTurn->sendDisplayMsg();

			// TODO: port this into a blocking bool in Display which we can check, will be more generic
			m_pausedForDialog = true;
			return false;
		}
		else
		{
			payAmount = ( tax ? tax : (int)(taxPercentage * pTurn->assets() / 100 ) );
			Estate *payTarget = es->payTarget();
				
			if (!pTurn->payMoney(payAmount))
			{
				// TODO: If we go into debt here, we'll never enter rent
				// calculation. So, estates with tax shouldn't be ownable
				// ever.
				setDisplay(es, false, false, "Game paused, %s owes %d but is not solvent. Player needs to raise %d in cash first.", pTurn->name().c_str(), payAmount, (payAmount - pTurn->money()));
				newDebt(pTurn, 0, payTarget, payAmount);
				return true;
			}
			else if (payTarget && m_collectFines)
				payTarget->addMoney(payAmount);

			setDisplay(es, false, false, "%s pays %d.", pTurn->name().c_str(), payAmount);
		}
	}

	// Some estates have cards. Handle them before we do rent and purchasing.
	if ( CardGroup *cardGroup = es->takeCardGroup() )
		endTurn = giveCard(pTurn, es, cardGroup->nextCard());

	// Calculate rent for owned estates or offer them for sale.
	if ( es->canBeOwned() )
	{
		if ((pOwner = es->owner()))
		{
			// Owned.
			
			if (pOwner == pTurn)
				setDisplay(es, false, false, "%s already owns it.", pTurn->name().c_str());
			else if (es->mortgaged())
				setDisplay(es, false, false, "%s pays no rent because it's mortgaged.", pTurn->name().c_str());
			else
			{
				// Pay payAmount owed.
				int payAmount = es->rent(pTurn, extraRent());
				setExtraRent(false);

				if (!pTurn->payMoney(payAmount))
				{
					setDisplay(es, false, false, "Game paused, %s owes %d to %s but is not solvent. Player needs to raise %d in cash first.", pTurn->name().c_str(), payAmount, pOwner->name().c_str(), (payAmount - pTurn->money()));
					newDebt(pTurn, pOwner, 0, payAmount);
					return true;
				}
				setDisplay(es, false, false, "%s pays %d rent to %s.", pTurn->name().c_str(), payAmount, pOwner->name().c_str());
				pOwner->addMoney(payAmount);
			}
		}
		else
		{
			// Unowned, thus for sale.
			Player *pTmp = 0;
			for (std::vector<Player *>::iterator it = m_players.begin() ; it != m_players.end() && (pTmp = *it) ; ++it)
				if (pTmp != pTurn)
				{
					pTmp->setDisplay(es, "For sale.");
					pTmp->sendDisplayMsg();
				}

			pTurn->setDisplay(es, "For sale.");
			pTurn->addDisplayButton(".eb", "Buy", 1);
			pTurn->addDisplayButton(".ea", "Auction", m_auctionsEnabled);
			pTurn->addDisplayButton(".E", "End Turn", !m_auctionsEnabled);
			pTurn->sendDisplayMsg();

			pTurn->setCanRoll(false);
			pTurn->setCanBuy(true);
			return false;
		}
	}
	return endTurn;
}
bool Game::giveCard(Player *pTurn, Estate *estate, Card *card)
{
	if (!card)
	{
		ioError(pTurn->name() + " should get a card, but there doesn't seem to be any available!");
		return true;
	}
	setDisplay(estate, false, false, "%s", card->name().c_str());

	// If canBeOwned, remove from stack and give to player. Card can still
	// have actions afterwards.
	if (card->canBeOwned())
		transferCard(card, pTurn);

	// Actions
	if (card->toJail())
	{
		if ( Estate *eTmp = findNextJailEstate(pTurn->estate()) )
			pTurn->toJail(eTmp);
		else
			ioError("This gameboard does not have a jail estate.");

		updateTurn();
	}
	else if (card->pay())
	{
		int payAmount = card->pay();
		if (payAmount > 0)
		{
			if (!pTurn->payMoney(payAmount))
			{
				setDisplay(estate, false, false, "Game paused, %s owes %d but is not solvent. Player needs to raise %d in cash first.", pTurn->name().c_str(), payAmount, (payAmount - pTurn->money()));
				newDebt(pTurn, 0, pTurn->estate()->payTarget(), payAmount);
				return false;
			}
			else if (pTurn->estate()->payTarget() && m_collectFines)
				pTurn->estate()->payTarget()->addMoney(payAmount);
		}
		else if (payAmount < 0)
			pTurn->addMoney(-payAmount);
	}
	else if (card->payEach())
	{
		Player *pTmp = 0;
		int payAmount = (m_players.size() - 1) * card->payEach();

		if (payAmount > 0)
		{
			if (!pTurn->payMoney(payAmount))
			{
				setDisplay(estate, false, false, "Game paused, %s owes %d in %d but is not solvent. Player needs to raise %d in cash first.", pTurn->name().c_str(), payAmount, (payAmount - pTurn->money()));
				newDebtToAll(pTurn, payAmount);
			}
			else
			{
				for (std::vector<Player *>::iterator it = m_players.begin() ; it != m_players.end() && (pTmp = *it) ; ++it)
				if (pTmp != pTurn)
					pTmp->addMoney(card->payEach());
			}
		}
		else
		{
			payAmount = -(card->payEach());
			for (std::vector<Player *>::iterator it = m_players.begin() ; it != m_players.end() && (pTmp = *it) ; ++it)
				if (pTurn != pTmp)
				{
					if (!pTmp->payMoney(payAmount))
					{
						setDisplay(estate, false, false, "Game paused, %s owes %d but is not solvent. Player needs to raise %d in cash first.", pTmp->name().c_str(), payAmount, (payAmount - pTmp->money()));
						newDebt(pTmp, pTurn, 0, payAmount);
					}
					else
						pTurn->addMoney(payAmount);
				}
			}
	}
	else if (card->payHouse() && card->payHotel())
	{
		int payAmount = 0;

		Estate *eTmp = 0;
		for (std::vector<Estate *>::iterator eIt = m_estates.begin() ; eIt != m_estates.end() && (eTmp = *eIt) ; eIt++)
		{
			if (pTurn == eTmp->owner())
			{
				if (eTmp->houses()==5)
					payAmount += card->payHotel();
				else if (eTmp->houses())
					payAmount += card->payHouse();
			}
		}

		if (payAmount > 0)
		{
			if (!pTurn->payMoney(payAmount))
			{
				setDisplay(estate, false, false, "Game paused, %s owes %d but is not solvent. Player needs to raise %d in cash first.", pTurn->name().c_str(), payAmount, (payAmount - pTurn->money()));
				newDebt(pTurn, 0, pTurn->estate()->payTarget(), payAmount);
				return false;
			}
			else if (pTurn->estate()->payTarget() && m_collectFines)
				pTurn->estate()->payTarget()->addMoney(payAmount);
		}
		else if (payAmount < 0)
			pTurn->addMoney(-payAmount);
	}
	else if (card->advance() || card->advanceTo()!=-1 || card->advanceToNextOf())
	{
		// Duplicated from cmd_roll. :-(
		if (card->advance())
			pTurn->advance(card->advance(), true);
		else if (card->advanceTo()!=-1)
			pTurn->advanceTo(card->advanceTo(), true);
		else if (EstateGroup *group = card->advanceToNextOf())
		{
			if (Estate *eTmp = findNextEstate(group, pTurn->estate()))
			{
				pTurn->advanceTo(eTmp->id(), true);
				setExtraRent(true);
			}
			else
				ioError("Could not find next estate on gameboard.");
		}
		ioWrite("<monopd><playerupdate playerid=\"%d\" location=\"%d\" directmove=\"1\"/></monopd>\n", pTurn->id(), pTurn->destination()->id());
		return landPlayer(pTurn, true);
	}
	return true;
}

void Game::declareBankrupt(Player *pInput)
{
	ioInfo(pInput->name() + " declares bankrupty!");
	bankruptPlayer(pInput);
}

void Game::bankruptPlayer(Player *pBroke)
{
	// Set bankrupt flag
	pBroke->setBankrupt(true);

	// TODO: Player can have multiple debts!

	// By the way, isn't this a large duplication of solveDebt which could
	// ignore a solvency check here?

	// Find debt and creditor
	Debt *debt = findDebt(pBroke);
	Estate *eCreditor = 0;
	Player *pCreditor = 0;
	if (debt)
	{
		eCreditor = debt->toEstate();
		pCreditor = debt->toPlayer();
	}

	// Give all cards to creditor or card stack
	while (Card *card = pBroke->findFirstCard())
		transferCard(card, pCreditor);

	// Sell all houses on property
	Estate *eTmp = 0;
	for(std::vector<Estate *>::iterator it = m_estates.begin() ; it != m_estates.end() && (eTmp = *it) ; ++it)
	{
		if (pBroke == eTmp->owner())
		{
			// Sell houses
			if (eTmp->houses())
			{
				// Get money
				int returnValue = eTmp->houses() * eTmp->housePrice() / 2;
				pBroke->addMoney(returnValue);

				// Make houses available again
				if (eTmp->houses()==5)
					setHotels(hotels() + 1);
				else
					setHouses(houses() + eTmp->houses());
				eTmp->setHouses(0);
			}

			// Transfer ownership to creditor
			if (pCreditor)
				transferEstate(eTmp, pCreditor);
			else
			{
				eTmp->setMortgaged(false);

				// TODO: auction all estate when there is no pCreditor
				transferEstate(eTmp, pCreditor);
			}
			// TODO: send estateupdate?
		}
	}

	// Give all money to creditor
	int payAmount = pBroke->money();
	pBroke->payMoney(payAmount);

	if (pCreditor)
	{
		pCreditor->addMoney(payAmount);
		ioWrite("<monopd><playerupdate playerid=\"%d\"/></monopd>\n", pCreditor->id());
	}
	else if (eCreditor)
	{
		eCreditor->addMoney(payAmount);
		ioWrite("<monopd><estateupdate estateid=\"%d\" money=\"%d\"/></monopd>\n", eCreditor->id(), eCreditor->money());
	}

	if (debt)
		delDebt(debt);

	// Send playerupdate for bankrupt player
	ioWrite("<monopd><playerupdate playerid=\"%d\" money=\"%d\" bankrupt=\"1\"/></monopd>\n", pBroke->id(), pBroke->money());

	// Update turn. Always when this function is called as command and
	// possibly necessary when this function is called for a
	// disconnected user.
	if (pBroke == m_pTurn)
		updateTurn();

	// Count active (non-bankrupt) players
	int activePlayers = 0;
	Player *pTmp = 0, *pWinner = 0;
	for(std::vector<Player *>::iterator it = m_players.begin(); (pTmp = *it) &&  it != m_players.end(); ++it)
		if (pTmp && !pTmp->bankrupt())
		{
			activePlayers++;
			pWinner = pTmp;
		}

	if (activePlayers == 1)
	{
		m_status = End;
		syslog(LOG_INFO, "game %d ended: %s wins with a fortune of %d.", m_id, pWinner->name().c_str(), pWinner->assets());

		pTmp = 0;
		for(std::vector<Player *>::iterator it = m_players.begin(); (pTmp = *it) &&  it != m_players.end(); ++it)
			if (pTmp)
			{
				pTmp->setDisplay(pWinner->estate(), "The game has ended! %s wins with a fortune of %d!", pWinner->name().c_str(), pWinner->assets());
				pTmp->addDisplayButton(".gx", "New Game", true);
				pTmp->sendDisplayMsg();
				sendStatus(pTmp);
			}
	}
	else
	{
		// Inform clients the game can continue again
		if (m_debts.size() == 0)
			ioInfo("All debts are settled, game continues.");
	}
}

int Game::lowestFreeId()
{
	int lowest = 0;

	// Find lowest free id in std::vector.
	Player *pTmp = 0;
	for(std::vector<Player *>::iterator it = m_players.begin(); (pTmp = *it) &&  it != m_players.end(); ++it)
		if (pTmp && pTmp->id() >= lowest)
			lowest = pTmp->id() + 1;

	return lowest;
}

void Game::rollDice()
{
	// TODO: decent dice display
	dice[0] = 1 + (int) (6.0 * rand()/(RAND_MAX + 1.0 ));
	dice[1] = 1 + (int) (6.0 * rand()/(RAND_MAX + 1.0 ));
}

unsigned int Game::players()
{
	return m_players.size();
}

unsigned int Game::connectedPlayers()
{
	unsigned int count = 0;
	Player *pTmp = 0;
	for(std::vector<Player *>::iterator it = m_players.begin(); it != m_players.end() && (pTmp = *it) ; ++it)
		if (pTmp->socket())
			count++;
	return count;
}
int Game::debts()
{
	return m_debts.size();
}

unsigned int Game::clientsMoving()
{
	unsigned int moving = 0;
	Player *pTmp = 0;
	for(std::vector<Player *>::iterator it = m_players.begin(); it != m_players.end() && (pTmp = *it) ; ++it) 
		if (!pTmp->bankrupt() && pTmp->tokenLocation())
			moving++;
	return moving;
}

void Game::setAllClientsMoving(Estate *estate)
{
	Player *pTmp = 0;
	for(std::vector<Player *>::iterator it = m_players.begin(); it != m_players.end() && (pTmp = *it) ; ++it)
	if (pTmp->socket())
		pTmp->setTokenLocation(estate);
}

void Game::sendStatus(Player *p)
{
	p->ioWrite("<monopd><gameupdate gameid=\"%d\" status=\"%s\"/></monopd>\n", m_id, statusLabel().c_str());
}

void Game::sendClientList(Player *p)
{
	// FIXME: DEPRECATED
	p->ioWrite("<monopd><updateplayerlist type=\"full\">");
	Player *pTmp = 0;
	for(std::vector<Player *>::iterator it = m_players.begin(); it != m_players.end() && (pTmp = *it) ; ++it)
		p->ioWrite("<player playerid=\"%d\" name=\"%s\" host=\"%s\" master=\"%d\"/>", pTmp->id(), pTmp->name().c_str(), pTmp->host().c_str(), (pTmp == m_master));
	p->ioWrite("</updateplayerlist></monopd>\n");

	p->ioWrite("<monopd>");
	pTmp = 0;
	for(std::vector<Player *>::iterator it = m_players.begin(); it != m_players.end() && (pTmp = *it) ; ++it)
		p->ioWrite("<playerupdate playerid=\"%d\" name=\"%s\" image=\"%s\" host=\"%s\" master=\"%d\"/>", pTmp->id(), pTmp->name().c_str(), pTmp->image().c_str(), pTmp->host().c_str(), (pTmp == m_master));
	p->ioWrite("</monopd>\n");
}

void Game::sendEstateList(Player *p)
{
	Estate *eTmp = 0;
	for(std::vector<Estate *>::iterator eIt = m_estates.begin(); eIt != m_estates.end() && (eTmp = *eIt) ; eIt++)
	{
		std::string bgColor;
		if (eTmp->bgColor().size())
			bgColor = eTmp->bgColor();
		else
		{
			EstateGroup *estateGroup = eTmp->group();
			if (estateGroup && estateGroup->bgColor().size())
				bgColor = estateGroup->bgColor();
			else
				bgColor = m_bgColor;
		}

		std::string color = "";
		if (eTmp->color().size())
			color = eTmp->color();
		else
		{
			EstateGroup *estateGroup = eTmp->group();
		 	if (estateGroup && estateGroup->color().size())
				color = estateGroup->color();
		}

		p->ioWrite("<estateupdate estateid=\"%d\" name=\"%s\" color=\"%s\" bgcolor=\"%s\" image=\"%s\" owner=\"%d\" houses=\"%d\" houseprice=\"%d\" sellhouseprice=\"%d\" mortgageprice=\"%d\" unmortgageprice=\"%d\" mortgaged=\"%d\" group=\"%d\" can_be_owned=\"%d\" can_toggle_mortgage=\"%d\" can_buy_houses=\"%d\" can_sell_houses=\"%d\" price=\"%ld\" rent0=\"%d\" rent1=\"%d\" rent2=\"%d\" rent3=\"%d\" rent4=\"%d\" rent5=\"%d\" money=\"%d\"/>", eTmp->id(), eTmp->name().c_str(), color.c_str(), bgColor.c_str(), eTmp->icon().c_str(), (eTmp->owner() ? eTmp->owner()->id() : -1), eTmp->houses(), eTmp->housePrice(), eTmp->sellHousePrice(), eTmp->mortgagePrice(), eTmp->unmortgagePrice(), eTmp->mortgaged(), eTmp->group() ? eTmp->group()->id() : -1, eTmp->canBeOwned(), eTmp->canToggleMortgage(p), eTmp->canBuyHouses(p), eTmp->canSellHouses(p), eTmp->price(), eTmp->rentByHouses(0), eTmp->rentByHouses(1), eTmp->rentByHouses(2), eTmp->rentByHouses(3), eTmp->rentByHouses(4), eTmp->rentByHouses(5), eTmp->money());
	}
}

void Game::sendEstateGroupList(Player *p)
{
	EstateGroup *egTmp = 0;
	for(std::vector<EstateGroup *>::iterator it = m_estateGroups.begin(); it != m_estateGroups.end() && (egTmp = *it) ; ++it)
		p->ioWrite("<estategroupupdate groupid=\"%d\" name=\"%s\"/>", egTmp->id(), egTmp->name().c_str());
}

void Game::sendPlayerList(Player *pOut, bool includeCards)
{
	Player *pTmp = 0;
	for(std::vector<Player *>::iterator it = m_players.begin() ; it != m_players.end() && (pTmp = *it) ; ++it)
	{
		pOut->ioWrite("<playerupdate playerid=\"%d\" name=\"%s\" image=\"%s\" money=\"%d\" location=\"%d\" jailed=\"0\" directmove=\"1\" hasturn=\"%d\" can_roll=\"%d\"/>", pTmp->id(), pTmp->name().c_str(), pTmp->image().c_str(), pTmp->money(), pTmp->estate()->id(), pTmp->hasTurn(), pTmp->canRoll());

		if (includeCards)
			pTmp->sendCardList(pOut);
	}
}

void Game::sendFullUpdate(Player *p, const bool userRequest)
{
	if (p->requestedUpdate())
	{
		// Last request was also a full update, so the full update XML
		// itself triggers the request. Return here to avoid race condition.
		return;
	}
	if (userRequest)
		p->setRequestedUpdate(true);

	sendStatus(p);

	if (m_status == Config)
		sendClientList(p);
	else if (m_status == Init || m_status == Run)
	{
		p->ioWrite("<monopd>");
		sendPlayerList(p, true); // includeCards
		sendEstateGroupList(p);
		sendEstateList(p);
		p->ioWrite("</monopd>\n");
		p->sendDisplayMsg();
	}
}

void Game::sendConfiguration(Player *p)
{
	// TODO: write sendConfigUpdate which only sends the updated entries
	bool edit = (p == m_master);
	p->ioWrite("<monopd><configupdate gameid=\"%d\"><option type=\"bool\" title=\"Free Parking collects fines\" command=\".gef\" value=\"%d\" edit=\"%d\"/><option type=\"bool\" title=\"Enable auctions\" value=\"%d\" command=\".gea\" edit=\"%d\"/></configupdate></monopd>\n", m_id, m_collectFines, edit, m_auctionsEnabled, edit);
	// p->ioWrite("<monopd><configupdate gameid=\"%d\"><option type=\"bool\" title=\"Free Parking collects fines\" command=\".gef\" value=\"%d\" edit=\"%d\"/><option type=\"bool\" title=\"Enable auctions\" value=\"%d\" command=\".gea\" edit=\"%d\"/><option type=\"bool\" title=\"Double pass money on exact landings\" command=\".gep\" value=\"%d\" edit=\"%d\"/></configupdate></monopd>\n", m_id, m_collectFines, edit, m_auctionsEnabled, edit, m_doublePassMoney, edit);
}

void Game::setDescription(const std::string desc)
{
	m_description = desc;
}

void Game::setCollectFines(const bool collectFines)
{
	if (m_collectFines != collectFines)
	{
		m_collectFines = collectFines;

		Player *pTmp = 0;
		for(std::vector<Player *>::iterator it = m_players.begin(); it != m_players.end() && (pTmp = *it) ; ++it)
			sendConfiguration(pTmp);
	}
}

void Game::setAuctionsEnabled(const bool auctionsEnabled)
{
	if (m_auctionsEnabled != auctionsEnabled)
	{
		m_auctionsEnabled = auctionsEnabled;

		Player *pTmp = 0;
		for(std::vector<Player *>::iterator it = m_players.begin(); it != m_players.end() && (pTmp = *it) ; ++it)
			sendConfiguration(pTmp);
	}
}

void Game::setDoublePassMoney(bool doublePassMoney)
{
	if (m_doublePassMoney != doublePassMoney)
	{
		m_doublePassMoney = doublePassMoney;

		Player *pTmp = 0;
		for(std::vector<Player *>::iterator it = m_players.begin(); it != m_players.end() && (pTmp = *it) ; ++it)
			sendConfiguration(pTmp);
	}
}

const std::string Game::statusLabel()
{
	switch(m_status)
	{
	case Config:
		return "config";
	case Init:
		return "init";
	case Run:
		return "run";
	case End:
		return "end";
	default:
		return "default";
	}
}

const bool Game::canBeJoined()
{
	return (m_players.size() < m_maxPlayers);
}
