// Copyright (c) 2001-2003 Rob Kaper <cap@capsi.com>,
//               2001 Erik Bourget <ebourg@cs.mcgill.ca>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// version 2 as published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; see the file COPYING.  If not, write to
// the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>

#include <string>

#include <libcapsinetwork/socket.h>

#include "card.h"
#include "const.h"
#include "display.h"
#include "estate.h"
#include "io.h"
#include "game.h"
#include "player.h"
#include "trade.h" // move trade commands to game/engine

Player::Player(Socket *socket, int id)
	: GameObject(id, "", GameObject::GPlayer)
{
	m_socket = socket;
	m_game = 0;
	m_display = new Display();
	m_cookie = std::string(itoa(id)) + "/" + itoa(rand());
	m_image = "";
	reset();
}

Player::~Player()
{
	delete m_display;
}

void Player::reset()
{
	m_estate = m_destination = m_tokenLocation = 0;
	m_money = 0;
	m_doubleCount = m_jailCount = 0;
	m_hasTurn = m_canRoll = m_canRollAgain = m_canBuy = m_jailed = m_bankrupt = false;
	m_requestedUpdate = false;
}

void Player::ioWrite(const char *fmt, ...)
{
	int n, size = 256;
	char *buf = new char[size];
	static std::string ioStr;
	va_list arg;

	buf[0] = 0;

	while (1)
	{
		va_start(arg, fmt);
		n = vsnprintf(buf, size, fmt, arg);
		va_end(arg);

		if (n > -1 && n < size)
		{
			ioStr = buf;
			delete[] buf;
			ioWrite(ioStr);
			return;
		}

		if (n > -1)
			size = n+1;
		else
			size *= 2;

		delete[] buf;
		buf = new char[size];
	}
}

void Player::ioWrite(std::string data)
{
	if (m_socket)
		m_socket->ioWrite(data);
}

void Player::ioInfo(const char *data, ...)
{
	va_list arg;
	char buf[2048];
	
	va_start(arg, data);
	vsnprintf(buf, sizeof(buf)-1, data, arg);
	va_end(arg);
	buf[sizeof(buf)-1] = 0;

	ioWrite("<monopd><msg type=\"info\" value=\"%s\"/></monopd>\n", buf);
}

void Player::ioInfo(const std::string data)
{
	ioWrite("<monopd><msg type=\"info\" value=\"" + data + "\"/></monopd>\n");
}

void Player::ioError(const char *data, ...)
{
	va_list arg;
	char buf[2048];
	
	va_start(arg, data);
	vsnprintf(buf, sizeof(buf)-1, data, arg);
	va_end(arg);
	buf[sizeof(buf)-1] = 0;

	ioWrite("<monopd><msg type=\"error\" value=\"%s\"/></monopd>\n", buf);
}

void Player::ioError(const std::string data)
{
	ioWrite("<monopd><msg type=\"error\" value=\"" + data + "\"/></monopd>\n");
}

void Player::ioNoSuchCmd(const std::string data)
{
	if (data.size())
		ioError("Command is unavailable at current game status: " + data);
	else
		ioError("No such command.");
}

void Player::setName(std::string name)
{
	m_name = name;
}

void Player::setImage(const std::string &img)
{
	if (m_image != img)
	{
		m_image = img;
		m_game->ioWrite("<monopd><playerupdate playerid=\"%d\" image=\"%s\"/></monopd>\n", m_id, image().c_str());
	}
}

void Player::setDisplay(Estate *estate, const char *data, ...)
{
	va_list arg;
	char buf[2048];
	
	va_start(arg, data);
	vsnprintf(buf, sizeof(buf)-1, data, arg);
	va_end(arg);
	buf[sizeof(buf)-1] = 0;

	m_display->setEstate(estate);
	m_display->setText(std::string(buf));
	m_display->resetButtons();
}

void Player::setDisplay(Estate *estate, const std::string data)
{
	m_display->setEstate(estate);
	m_display->setText(data);
	m_display->resetButtons();
}

void Player::setDisplayClearText(bool clearText)
{
	m_display->setClearText(clearText);
}

void Player::setDisplayClearButtons(bool clearButtons)
{
	m_display->setClearButtons(clearButtons);
}

void Player::addDisplayButton(const std::string command, const std::string caption, const bool enabled)
{
	m_display->addButton(command, caption, enabled);
}

void Player::sendDisplayMsg()
{
	Estate *estate = m_display->estate();
	if (estate)
	{
		std::vector<DisplayButton *> buttons = m_display->buttons();
		if (buttons.size() > 0)
		{
			ioWrite("<monopd><display estateid=\"%d\" text=\"%s\" cleartext=\"%d\" clearbuttons=\"%d\">", estate->id(), m_display->text().c_str(), m_display->clearText(), m_display->clearButtons());
			DisplayButton *button = 0;
			for (std::vector<DisplayButton *>::iterator it = buttons.begin() ; it != buttons.end() && (button = *it) ; ++it)
				ioWrite("<button command=\"%s\" caption=\"%s\" enabled=\"%d\"/>", button->command().c_str(), button->caption().c_str(), button->enabled());
			ioWrite("</display></monopd>\n", estate->id(), m_display->text().c_str());
		}
		else
			ioWrite("<monopd><display estateid=\"%d\" text=\"%s\" cleartext=\"%d\" clearbuttons=\"%d\"/></monopd>\n", estate->id(), m_display->text().c_str(), m_display->clearText(), m_display->clearButtons());
	}
	else
		ioWrite("<monopd><display estateid=\"-1\" text=\"%s\" cleartext=\"%d\" clearbuttons=\"%d\"/></monopd>\n", m_display->text().c_str(), m_display->clearText(), m_display->clearButtons());

	m_display->setClearText(false);
	m_display->setClearButtons(false);
}

void Player::sendClientMsg()
{
	ioWrite(std::string("<monopd><server version=\"") + MONOPD_VERSION_STRING + "\"/><client playerid=\"" + itoa(m_id) + "\" cookie=\"" + m_cookie.c_str() + "\"/></monopd>\n");
}

void Player::sendCardList(Player *pOut)
{
	Card *card = 0;
	for(std::vector<Card *>::iterator it = m_cards.begin() ; it != m_cards.end() && (card = *it) ; ++it)
		pOut->ioWrite("<cardupdate cardid=\"%d\" title=\"%s\" owner=\"%d\"/>", card->id(), card->name().c_str(), m_id);
}

void Player::rollDice()
{
	if (m_jailed)
	{
		ioError("You cannot roll while jailed.");
		return;
	}
	if (m_canBuy)
	{
		ioError("You must buy or auction the current estate.");
		return;
	}

	m_game->rollDice();
	m_game->setDisplay(0, true, false, "%s rolls %d and %d.", name().c_str(), m_game->dice[0], m_game->dice[1]);

	// Take away many privileges
	setCanRoll(false);
	setCanRollAgain(false);
	setCanBuy(false);

	if (m_game->dice[0] == m_game->dice[1])
	{
		if(++m_doubleCount < 3)
			setCanRollAgain(true);
		else
		{
			m_game->ioWrite("<monopd><msg type=\"info\" value=\"%s throws three doubles and is thrown into jail.\"/></monopd>\n", name().c_str());

			if (Estate *eJail = m_game->findNextJailEstate(m_estate))
				toJail(eJail);
			else
				m_game->ioError("This gameboard does not have a jail estate.");

			m_game->updateTurn();
			return;
		}
	}
	else
		m_doubleCount = 0;

	// Update player board position and nothing more. Other player
	// properties are not actually updated yet, this happens when all
	// clients have landed the player on the destination estate. Players do
	// get asynchronous updates, these are calculated per client.
	advance(m_game->dice[0] + m_game->dice[1], false);

	// Send message to clients to move token. The rest is done by token confirmation or timeout.
	m_game->ioWrite("<monopd><playerupdate playerid=\"%d\" location=\"%d\" directmove=\"0\"/></monopd>\n", m_id, m_destination->id());
}

void Player::endTurn(bool userRequest)
{
	if (!m_hasTurn)
		return;

	if (m_game->findDebt(this))
    {
    	if (userRequest)
    		ioError("You have a debt, cannot end turn.");
    	return;
    }

	if (m_canBuy)
	{
		if (m_game->auctionsEnabled())
		{
			if (userRequest)
				ioError("You cannot end your turn, you must either buy or auction the property you are on.");
			return;
		}
		else if (!userRequest)
			return;
	}
	else if (m_canRollAgain)
	{
		m_game->setDisplay(m_estate, false, false, "%s may roll again.", name().c_str());
		
		setCanRoll(true);
		setCanRollAgain(false);
		return;
	}
	else if (m_canRoll)
	{
		if (userRequest)
			ioError("You cannot end your turn, you must roll first!");
		return;
	}
	else if (m_jailed)
	{
		if (userRequest)
			ioError("You cannot end your turn while jailed.");
		return;
	}
	
	else if (m_game->pausedForDialog())
	{
		if (userRequest)
			ioError("You must answer the dialog first.");
		return;
	}

	// Turn goes to next player
	m_game->updateTurn();
}

void Player::payJail()
{
	int payAmount = m_estate->payAmount();
	if (!payMoney(payAmount))
	{
		ioError("Leaving jail costs %d, you only have %d.", payAmount, m_money);
		return;
	}
	else if (m_estate->payTarget() && m_game->collectFines())
		m_estate->payTarget()->addMoney(payAmount);

	setJailed(false);
	setCanRoll(true);
	m_jailCount = 0;
	m_game->ioWrite("<monopd><playerupdate playerid=\"%d\" jailed=\"0\"/></monopd>\n", m_id);
	m_game->setDisplay(m_estate, false, true, "%s paid %d and has left jail, can roll now.", name().c_str(), payAmount);
}

void Player::rollJail()
{
	m_game->rollDice();

	// Leave jail for free when doubles are rolled
	if (m_game->dice[0] == m_game->dice[1])
	{
		setJailed(false);
		m_game->ioWrite("<monopd><playerupdate playerid=\"%d\" jailed=\"0\"/></monopd>\n", m_id);
		m_game->setDisplay(m_estate, false, true, "Doubles, leaving jail!");

		advance(m_game->dice[0] + m_game->dice[1], false);

		// Send message to clients to move token. The rest is done by the token confirmation or timeout.
		m_game->ioWrite("<monopd><playerupdate playerid=\"%d\" location=\"%d\" directmove=\"0\"/></monopd>\n", m_id, m_destination->id());

		return;
	}

	// No doubles, stay in jail the first three turns
	if(m_jailCount <= 3)
	{
		m_game->setDisplay(m_estate, false, true, "No doubles, staying in jail.");
		m_jailCount++;
		m_game->updateTurn();
		return;
	}

	// Third turn in jail, no doubles after three turns, must pay and leave.
	int payAmount = m_estate->payAmount();
	setJailed(false);
	m_game->ioWrite("<monopd><playerupdate playerid=\"%d\" jailed=\"0\"/></monopd>\n", m_id);
	m_game->setDisplay(0, false, true, "No doubles, %s must pay %d now and leave jail.", name().c_str(), payAmount);
	if (!payMoney(payAmount))
	{
		m_game->ioInfo("Game paused, %s owes %d but is not solvent. Player needs to raise %d in cash first.", name().c_str(), payAmount, (payAmount - m_money));
		setCanRollAgain(true); // player can roll once debt is resolved
		m_game->newDebt(this, 0, m_estate->payTarget(), payAmount);
		return;
	}

	if (m_estate->payTarget() && m_game->collectFines())
		m_estate->payTarget()->addMoney(payAmount);

	setCanRoll(true);
	m_jailCount = 0;
}

void Player::useJailCard()
{
	Card *card = findOutOfJailCard();
	if (!card)
	{
		ioError("You don't have any 'out of jail' cards.");
		return;
	}
	m_game->transferCard(card, 0);

	setCanRoll(true);
	setJailed(false);
	m_jailCount = 0;
	m_game->ioWrite("<monopd><playerupdate playerid=\"%d\" jailed=\"0\"/></monopd>\n", m_id);
	m_game->setDisplay(0, false, true, "%s used card and has left jail, can roll now.", name().c_str());
}

void Player::buyEstate()
{
	if (!m_canBuy)
	{
		ioError("You cannot buy anything at the moment.");
		return;
	}

	if (!payMoney(m_estate->price()))
	{
		ioError("You do not have enough money to buy this property, %s costs %d.", m_estate->name().c_str(), m_estate->price());
		return;
	}

	m_game->transferEstate(m_estate, this);
	m_game->setDisplay(m_estate, false, true, "Purchased by %s for %d.", name().c_str(), m_estate->price());
	setCanBuy(false);
	endTurn();
}

void Player::mortgageEstate(int estateId)
{
	Estate *estate = m_game->findEstate(estateId);
	if (!estate)
	{
		ioError("No such estateId: %d.", estateId);
		return;
	}

	if (this != estate->owner())
	{
		ioError("You don't own '" + estate->name() + "'.");
		return;
	}

	if (estate->mortgaged())
	{
		if (!payMoney(estate->unmortgagePrice()))
		{
			ioError("'Unmortgaging %s' costs %d, you only have %d.", estate->name().c_str(), estate->unmortgagePrice(), m_money);
			return;
		}
		estate->setMortgaged(false);
		m_game->sendMsgEstateUpdate(estate);
		m_game->ioInfo("%s unmortgaged %s for %d.", name().c_str(), estate->name().c_str(), estate->unmortgagePrice());
	}
	else
	{
		// Mortgage the property, if possible!

		if (estate->groupHasBuildings())
		{
			ioError("You can't mortgage '" + estate->name() + "',  there are still houses in the same group.");
			return;
		}

		estate->setMortgaged(true);
		m_game->sendMsgEstateUpdate(estate);

		addMoney((estate->mortgagePrice()));
		m_game->ioInfo("%s mortgaged %s for %d.", name().c_str(), estate->name().c_str(), estate->mortgagePrice());
		m_game->solveDebt(this);

		endTurn();
	}
}

void Player::payTax(const bool percentage)
{
	if (!m_hasTurn)
		return;

	m_game->setPausedForDialog(false);

	int payAmount = 0;
	if (percentage)
		payAmount = (int)( m_estate->taxPercentage() * assets() / 100 );
	else
		payAmount = m_estate->tax();

	m_game->setDisplay(m_estate, false, false, "%s chooses to pay the %s tax, which amounts to %d.", name().c_str(), (percentage ? "assets based" : "static"), payAmount);

	if (!payMoney(payAmount))
	{
		m_game->ioInfo("Game paused, %s owes %d but is not solvent. Player needs to raise %d in cash first.", name().c_str(), payAmount, (payAmount - m_money));
		m_game->newDebt(this, 0, m_estate->payTarget(), payAmount);
		return;
	}

	if (m_estate->payTarget() && m_game->collectFines())
		m_estate->payTarget()->addMoney(payAmount);

	endTurn();
}

void Player::buyHouse(int estateId)
{
	Estate *estate = m_game->findEstate(estateId);
	if (!estate)
	{
		ioError("No such estateId: %d.", estateId);
		return;
	}

	if (this != estate->owner())
	{
		ioError("You don't own '" + estate->name() + "'.");
		return;
	}
	else if (!estate->housePrice())
	{
		ioError("You can't build houses on '" + estate->name() + "'.");
		return;
	}
	else if (!estate->estateGroupIsMonopoly())
	{
		ioError("You don't own all estate in the group of '" + estate->name() + "'.");
		return;
	}
	else if (estate->groupHasMortgages())
	{
		ioError("One or more estates in the group of '" + estate->name() + "' are mortgaged, so you cannot build houses there.");
		return;
	}
	else if (estate->houses()==5)
	{
		ioError("You already have a hotel on '" + estate->name() + "'.");
		return;
	}
	else if (estate->houses() >= estate->maxHouses()) 
	{
		ioError("The even build rule prevents you from building more house on '" + estate->name() + "'.");
		return;
	}

	if (estate->houses() < 4)
	{
		if (!m_game->houses())
		{
			ioError("Sorry, there are no more houses available!");
			return;
		}
	}
	else if (!m_game->hotels())
	{
		ioError("Sorry, there are no more hotels available!");
		return;
	}

	int cost = estate->housePrice();
	if (!payMoney(cost))
	{
		ioError("A house for %s costs %d, you only have %d.", estate->name().c_str(), cost, money());
		return;
	}

	// Wow, we met all the right criteria? Let's build then!
	estate->setHouses(estate->houses() + 1);
	if (estate->houses()==5)
	{
		m_game->setHotels(m_game->hotels() - 1);
		m_game->setHouses(m_game->houses() + 4);
		m_game->ioInfo(name() + " buys a hotel for '" + estate->name() + "'.");
	}
	else
	{
		m_game->setHouses(m_game->houses() - 1);
		m_game->ioInfo(name() + " buys a house for '" + estate->name() + "'.");
	}

	m_game->sendMsgEstateUpdate(estate);
}

void Player::sellHouse(int estateId)
{
	Estate *estate = m_game->findEstate(estateId);
	if (!estate)
	{
		ioError("No such estateId: %d.", estateId);
		return;
	}

	if (this != estate->owner())
	{
		ioError("You don't own '" + estate->name() + "'.");
		return;
	}
	else if (estate->houses()==0)
	{
		ioError("There are no buildings on '" + estate->name() + "' to sell.");
		return;
	}
	else if (estate->houses() <= estate->minHouses())
	{
		ioError("You must level out the number of houses across your monopoly before you sell more on '" + estate->name() + "'.");
		return;
	}

	if ((estate->houses() == 5) && (m_game->houses() < 4))
	{
		ioError("Sorry, there are no more houses available!");
		return;
	}

	// Wow, we met all the right criteria? Let's sell!
	addMoney(estate->sellHousePrice());
	estate->setHouses(estate->houses() - 1);
	if (estate->houses()==4)
	{
		m_game->setHotels(m_game->hotels() + 1);
		m_game->setHouses(m_game->houses() - 4);
		m_game->ioInfo(name() + " sells a hotel for 4 houses on '" + estate->name() + "'.");
	}
	else
	{
		m_game->setHouses(m_game->houses() + 1);
		m_game->ioInfo(name() + " sells a house from '" + estate->name() + "'.");
	}

	m_game->sendMsgEstateUpdate(estate);
	m_game->solveDebt(this);
	endTurn();
}

void Player::updateTradeObject(char *data)
{
	bool useEstate = false;
	bool useCard = false;
	if (data[0] == 'c')
		useCard=true;
	else if (data[0] == 'e')
		useEstate = true;
	else
		return;

	data++;

	// data looks like "1:10:1", tradeid, objectid, playerid
	if (!strstr(data, ":"))
	{
		ioError("Invalid input, no seperator after tradeId");
		return;
	}
	int tradeId = atoi(strsep(&data, ":"));

	if (!strstr(data, ":"))
	{
		ioError("Invalid input, no seperator after objectId");
		return;
	}
	int objectId = atoi(strsep(&data, ":"));
	int playerId = atoi(data);

	Trade *trade = m_game->findTrade(tradeId);
	if (!trade)
	{
		ioError("No such tradeId: %d.", tradeId);
		return;

	}
	if (!trade->hasPlayer(this))
	{
		ioError("You are not part of trade %d.", tradeId);
		return;
	}

	GameObject *object = 0;
	Player *pFrom = 0;
	if (useEstate)
	{
		Estate *estate = m_game->findEstate(objectId);
		if (!estate)
		{
			ioError("No estate with estateId %d.", objectId);
			return;
		}
		if (estate->groupHasBuildings())
		{
			ioError("You can't trade " + estate->name() + ",  there are houses in the same group.");
			return;
		}
		object = estate;
		pFrom = estate->owner();
		if (!pFrom)
			return;
	}
	else if (useCard)
	{
		Card *card = m_game->findCard(objectId);
		if (!card || !card->owner())
		{
			ioError("Noone owns a card with cardId %d.", objectId);
			return;
		}
		object = card;
		pFrom = this;
	}

	// NULL player allowed, Trade will delete the component.
	Player *playerTo = m_game->findPlayer(playerId);

	trade->updateObject(object, pFrom, playerTo);
}

void Player::updateTradeMoney(char *data)
{
	// data looks like "1:1:1:100", tradeid, playerfrom, playerto, money
	if (!strstr(data, ":"))
	{
		ioError("Invalid input for .Tm, no seperator after tradeId");
		return;
	}
	int tradeId = atoi(strsep(&data, ":"));

	if (!strstr(data, ":"))
	{
		ioError("Invalid input for .Tm, no seperator after playerFromId");
		return;
	}
	int playerFromId = atoi(strsep(&data, ":"));

	if (!strstr(data, ":"))
	{
		ioError("Invalid input for .Tm, no seperator after playerToId");
		return;
	}

	int playerToId = atoi(strsep(&data, ":"));
	unsigned int money = atoi(data);

	Trade *trade = m_game->findTrade(tradeId);
	if (!trade)
	{
		ioError("No such tradeId: %d.", tradeId);
		return;
	}

	if (!trade->hasPlayer(this))
	{
		ioError("You are not part of trade %d.", tradeId);
		return;
	}

	Player *pFrom = m_game->findPlayer(playerFromId);
	if (!pFrom)
	{
		ioError("No such playerFromId: %d.", playerFromId);
		return;
	}
	Player *pTo = m_game->findPlayer(playerToId);
	if (!pTo)
	{
		ioError("No such playerToId: %d.", playerToId);
		return;
	}

	// TODO: should also count money player owes in other trade components
	if(pFrom->money() < money)
	{
		ioError("%s doesn't have %d to offer.", pFrom->name().c_str(), money);
		return;
	}

	GameObject *object = trade->findMoneyObject(pFrom, pTo);
	if (object)
		object->setId(money);
	else
		object = new GameObject(money, "", GameObject::Money);

	trade->updateObject(object, pFrom, pTo);
}

void Player::advance(int pos, bool direct)
{
	int estates = m_game->estates();
	int location = m_estate ? m_estate->id() : 0;
	
	location += pos;
	if (location >= estates || location < 0)
		location %= estates;

	m_destination = m_game->findEstate(location);
	if (!direct)
		m_game->setAllClientsMoving(m_estate);
}

void Player::advanceTo(int pos, bool direct)
{
	m_destination = m_game->findEstate(pos);
	if (!direct)
		m_game->setAllClientsMoving(m_estate);
}

void Player::toJail(Estate *jailEstate)
{
	m_estate = jailEstate;
	m_doubleCount = 0;
	m_jailCount = 0;
	setCanRoll(false);
	m_jailed = true;

	m_game->ioWrite("<monopd><playerupdate playerid=\"%d\" location=\"%d\" jailed=\"1\" directmove=\"1\"/></monopd>\n", m_id, m_estate->id());
}

void Player::addMoney(int amt)
{
	m_money += amt;
	m_game->ioWrite("<monopd><playerupdate playerid=\"%d\" money=\"%d\"/></monopd>\n", m_id, m_money);
}

bool Player::payMoney(unsigned int amount)
{
	if (m_money < amount)
		return false;

	m_money -= amount;
	m_game->ioWrite("<monopd><playerupdate playerid=\"%d\" money=\"%d\"/></monopd>\n", m_id, m_money);
	return true;
}

const std::string Player::host()
{
	return ((m_socket && m_socket->status() == Socket::Ok) ? m_socket->fqdn() : "");
}

void Player::closeSocket()
{
	m_socket->setStatus(Socket::Close);
}

bool Player::hasDebt() 
{
	if(m_game->findDebt(this))
		return true;
	return false;
}

void Player::setHasTurn(const bool hasTurn)
{
	if (m_hasTurn != hasTurn)
	{
		m_hasTurn = hasTurn;
		m_game->ioWrite("<monopd><playerupdate playerid=\"%d\" hasturn=\"%d\"/></monopd>\n", m_id, m_hasTurn);
	}

	setCanBuy(false);

	if (m_hasTurn)
	{
		if (m_jailed)
		{
			setDisplay(m_estate, "You are in jail, turn %d.", ++m_jailCount);
			addDisplayButton(".jp", "Pay", true);
			addDisplayButton(".jc", "Use card", findOutOfJailCard());
			addDisplayButton(".jr", "Roll", true);
			sendDisplayMsg();
		}
		else
			setCanRoll(true);
	}
	else
	{
		setCanRoll(false);
		setCanRollAgain(false);
	}
}

void Player::setCanRoll(bool canRoll)
{
	if (m_canRoll != canRoll)
	{
		m_canRoll = canRoll;
		ioWrite("<monopd><playerupdate playerid=\"%d\" can_roll=\"%d\"/></monopd>\n", m_id, m_canRoll);
	}
}

void Player::setCanRollAgain(bool canRollAgain)
{
	if (m_canRollAgain != canRollAgain)
		m_canRollAgain = canRollAgain;
}

void Player::setCanBuy(bool canBuy)
{
	if (m_canBuy != canBuy)
	{
		m_canBuy = canBuy;
		ioWrite("<monopd><playerupdate playerid=\"%d\" can_buyestate=\"%d\"/></monopd>\n", m_id, m_canBuy);
	}
}

void Player::setEstate(Estate *estate)
{
	if (m_estate != estate)
		m_estate = estate;
}

Estate *Player::estate()
{
	return m_estate;
}

void Player::setDestination(Estate *estate)
{
	if (m_destination != estate)
		m_destination = estate;
}

Estate *Player::destination()
{
	return m_destination;
}

void Player::setTokenLocation(Estate *estate)
{
	m_tokenLocation = estate;
}

Estate *Player::tokenLocation()
{
	return m_tokenLocation;
}

void Player::addCard(Card *card)
{
	m_cards.push_back(card);
}

Card *Player::findFirstCard()
{
	if (m_cards.size() > 0)
		return m_cards[0];

	return 0;
}

void Player::takeCard(Card *card)
{
	for(std::vector<Card *>::iterator it = m_cards.begin() ; it != m_cards.end() && (*it) ; ++it)
		if (*it == card)
		{
			m_cards.erase(it);
			return;
		}
}

Card *Player::findCard(unsigned int cardId)
{
	Card *card = 0;
	for(std::vector<Card *>::iterator it = m_cards.begin() ; it != m_cards.end() && (card = *it) ; ++it)
		if (card->id() == cardId)
			return card;
	return 0;
}

Card *Player::findOutOfJailCard()
{
	Card *card = 0;
	for(std::vector<Card *>::iterator it = m_cards.begin() ; it != m_cards.end() && (card = *it) ; ++it)
		if (card->outOfJail())
			return card;
	return 0;
}

unsigned int Player::assets()
{
	return m_game->playerAssets(this);
}
